﻿using UnityEngine;

public class BoxControl : MonoBehaviour
{
    [SerializeField] Camera mainCamera;
    Vector2 sizeCamera;

    void Start()
    {
        float orthographicSize = mainCamera.orthographicSize;
        sizeCamera = new Vector2(orthographicSize * Screen.width / Screen.height,
                                 orthographicSize);
    }

    void OnTriggerExit2D(Collider2D other)
    {
        Transform otherTransform = other.gameObject.transform;
        if (!other.CompareTag("Player") &&
            (otherTransform.position.x < mainCamera.transform.position.x - sizeCamera.x ||
             otherTransform.position.x > mainCamera.transform.position.x + sizeCamera.x ||
             otherTransform.position.y > mainCamera.transform.position.y + sizeCamera.y ||
             otherTransform.position.y < mainCamera.transform.position.y - sizeCamera.y))
            Destroy(other.gameObject);
    }

    void OnCollisionEnter2D(Collision2D other)
    {
        var obj = other.gameObject;

        if (!obj.CompareTag("Player")) return;
        var player = obj.GetComponent<PlayerController>();
        var playerRigidbody = player.GetComponent<Rigidbody2D>();

        Vector2 velocity = player.lastVelocity;

        if (other.transform.position.y < 5 - other.transform.GetComponent<SpriteRenderer>().size.y)
        {
            velocity.x *= -1;
            velocity.y *= 5;
        }
        else
        {
            velocity.y *= -0.5f;
        }

        // stop moving
        playerRigidbody.velocity = Vector2.zero;
        playerRigidbody.angularVelocity = 0;

        // add new computed force
        playerRigidbody.AddForce(velocity * 20);
    }
}