﻿using DefaultNamespace;
using UnityEngine;

public class CameraControl : MonoBehaviour
{
    [SerializeField] Vector3 defaultVectorCamera = new Vector3(4.66f - 0.23f, 1.3f, -10);
    [SerializeField] PlatformsControl platformsControl;
    bool movingCamera, creatingPlatform;
    Vector3 endPos;
    
    [SerializeField] float Velocity;
    [SerializeField] float MinDistance, DistanceToCreatePlatform;
    [SerializeField] GameManager game;
    
    public static Vector2 SizeCamera { get; private set; }

    void Awake()
    {
        float orthographicSize = GetComponent<Camera>().orthographicSize;
        SizeCamera = new Vector2(orthographicSize * Screen.width / Screen.height,
                                 orthographicSize);
    }

    void Start()
    {
        StabilizeCamera();
    }

    public void StabilizeCamera()
    {
        movingCamera = creatingPlatform = true;
        endPos = game.player.transform.position + defaultVectorCamera;
        endPos.z = -10;
    }

    void LateUpdate()
    {
        if (!movingCamera) return;
        Vector3 targetPos = endPos;
        float distance = Vector3.Distance(transform.position, targetPos);

        if (creatingPlatform && distance < DistanceToCreatePlatform)
        {
            creatingPlatform = false;
            platformsControl.CreateNewPlatforms(endPos);
        }
        if (distance < MinDistance)
        {
            transform.position = endPos;
            movingCamera = false;
        }
        Vector3 newPos = Vector3.Lerp(transform.position, targetPos, Velocity * Time.fixedDeltaTime);
        transform.Translate(transform.InverseTransformPoint(newPos));
    }
}