using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;
using Random = System.Random;

namespace DefaultNamespace
{
    public class GameManager : MonoBehaviour
    {
        [Header("Core")] public PlayerController player;
        public Camera camera;
        public Canvas ui;
        [SerializeField] GameObject WinScreenLayer;
        [SerializeField] GameObject DeathScreenLayer;
        public GameObject Foods;

        [SerializeField] Text phrases;
        [SerializeField] string[] cheeseMore;
        [SerializeField] string[] cabbageMore;
        [SerializeField] string[] tomatoMore;
        [SerializeField] string[] idealSandwich;
        readonly Random rand = new Random();
        
        [Header("Game Stats")] public int TargetIncreaseValue;
        public GameObject Cheese, Cabbage, Tomato, Dishes;
        public int[] Scores = new int[4], Targets = new int[4];

        bool isWin, isDeath;
        
        FoodScore[] scores;

        void Awake()
        {
            //if (scores != null) return;
            
            var names = new[] {"Cheese", "Cabbage", "Tomato", "Dishes"};
            scores = new[] {Cheese, Cabbage, Tomato, Dishes}
                .Select((t, i) => new FoodScore {name = names[i], sprite = t, score = Scores[i], target = Targets[i]})
                .ToArray();
        }

        public void KillPlayer()
        {
            if(!isWin){
                DeathScreenLayer.SetActive(true);
                player.OnKilled();
                isDeath = true;
            }
        }

        public void AddScore(string name)
        {
            scores.First(it => it.name == name).score++;
            UpdateScore();
            if (scores.All(it => it.score >= it.target))
                OnSandwitchReady();
        }

        public void AddScore(FoodControl.Foods food) => AddScore(food.ToString());

        void UpdateScore() =>
            scores
                .ToList()
                .ForEach(it =>
                    it.sprite.transform.GetChild(0).GetComponent<Text>().text =
                        it.target > 0 ? $"{it.score}/{it.target}" : it.score.ToString());

        void OnSandwitchReady()
        {
            if(!isDeath)
            {
                WinScreenLayer.SetActive(true);
                isWin = true;

                if (scores[0].score > scores[1].score && scores[0].score > scores[2].score)
                    phrases.text = cheeseMore[rand.Next(0, cheeseMore.Length)];
                else if (scores[1].score > scores[0].score && scores[1].score > scores[2].score)
                    phrases.text = cabbageMore[rand.Next(0, cabbageMore.Length)];
                else if (scores[2].score > scores[0].score && scores[2].score > scores[1].score)
                    phrases.text = tomatoMore[rand.Next(0, tomatoMore.Length)];
                else phrases.text = idealSandwich[rand.Next(0, idealSandwich.Length)];
            }
        }

        public void TryAgain() => SceneManager.LoadScene("SampleScene");
        
        public void Exit() => Application.Quit();

        public void ArcadeMode() => WinScreenLayer.SetActive(false);

        public void NextLevel()
        {
            scores
                .ToList()
                .ForEach(it =>
                {
                    it.target += it.target > 0 ? TargetIncreaseValue : 0;
                    it.score = 0;
                });
            
            UpdateScore();
            WinScreenLayer.SetActive(false);
        }
    }

    public class FoodScore
    {
        public string name;
        public GameObject sprite;
        public int score, target;
    }
}