using JetBrains.Annotations;
using UnityEngine;

namespace DefaultNamespace
{
    public class PlatformController : MonoBehaviour
    {
        public GameManager game;
        [SerializeField] bool KillPlayerWhenTouch;

        [CanBeNull] static PlatformController lastPlatform;
        Collider2D collider;

        void Awake()
        {
            collider = GetComponent<BoxCollider2D>();
            if (collider == null) collider = GetComponent<PolygonCollider2D>();
            collider.isTrigger = true;
            GetComponent<SpriteRenderer>().sortingOrder = 1;
        }

        void OnTriggerEnter2D(Collider2D other)
        {
            var obj = other.gameObject;

            if (!obj.CompareTag("Player")) return;

            var player = obj.GetComponent<PlayerController>();

            if (lastPlatform != this)
                player.OnMovedToPlatform(this);

            if (lastPlatform != null)
                lastPlatform.Invalidate();
            lastPlatform = this;

            if (CalculateCollisionAltitude(other) <= 0) return;
            collider.isTrigger = false;
            
            if (KillPlayerWhenTouch)
                game.KillPlayer();

            player.StopMoving();
        }

        void OnTriggerExit2D(Collider2D other)
        {
            if(other.CompareTag("Player"))
                Invalidate();
        }

        float CalculateCollisionAltitude(Collider2D collisionCollider)
        {
            Vector3 player = collisionCollider.bounds.center;

//            if (player.y > transform.position.y)
//                return 1;

            if (player.y < transform.position.y ||
                player.x < transform.position.x - GetComponent<SpriteRenderer>().sprite.rect.center.x / 100 ||
                player.x > transform.position.x + GetComponent<SpriteRenderer>().sprite.rect.center.x / 100)
                return -1;

            return 1;
        }

        public void Invalidate()
        {
            collider.isTrigger = true;
        }
    }
}