﻿using System;
using System.Collections.Generic;
using DefaultNamespace;
using UnityEngine;
using UnityEngine.UI;
using Random = System.Random;

public class PlayerController : MonoBehaviour
{
    [SerializeField] GameManager game;
    [SerializeField] float MinMouseRadius = 5f;
    [SerializeField] float MaxMouseRadius = 100f;
    [SerializeField] GameObject aimingScale;

    [SerializeField] GameObject Cheese, Cabbage, Tomato;
    
    float defaultScreen = 1200;
    
    [SerializeField] float Force = 10f;

    Rigidbody2D rigidbody;
    Vector2 startMousePosition, startMousePos;
    CameraControl cameraController;
    bool isAiming, inAir;
    
    [SerializeField] Sprite[] skins;

    void Start()
    {
        Random rand = new Random();
        transform.GetComponent<SpriteRenderer>().sprite = skins[rand.Next(0, skins.Length)];
    }
    
    //TODO опасно для жизни
    readonly Queue<Vector2> lastVelocitys = new Queue<Vector2>();
    public Vector2 lastVelocity;

    void Update()
    {
        lastVelocity = lastVelocitys.Dequeue();
        lastVelocitys.Enqueue(rigidbody.velocity);

        if (isAiming)
        {
            aimingScale.transform.position = new Vector2(GetMouseX(), GetMouseY());
            aimingScale.transform.eulerAngles = Vector3.zero;
            Vector2 direction = (startMousePos - (Vector2)aimingScale.transform.position) /
                                (aimingScale.transform.childCount + 1);
            aimingScale.transform.localPosition = Vector3.zero;
            for(int i = 0; i < aimingScale.transform.childCount; i++)
                aimingScale.transform.GetChild(i).localPosition = -direction * (i + 1);
        }
    }
    
    void Awake()
    {
        rigidbody = GetComponent<Rigidbody2D>();
        cameraController = game.camera.GetComponent<CameraControl>();
        for (int i = 0; i < 5; i++)
            lastVelocitys.Enqueue(Vector2.zero);
    }

    void OnMouseDown()
    {
        isAiming = true;
        startMousePosition = Input.mousePosition;
        startMousePos = new Vector2(GetMouseX(), GetMouseY());
        aimingScale.SetActive(true);
    }

    static float GetMouseX() => (Input.mousePosition.x * 2 / Screen.width - 1f) * CameraControl.SizeCamera.x;

    static float GetMouseY() => (Input.mousePosition.y * 2 / Screen.height - 1f) * CameraControl.SizeCamera.y;

    void OnMouseUp()
    {
        isAiming = false;
        aimingScale.SetActive(false);
        
        if (lastVelocity == rigidbody.velocity)
            Shoot(startMousePosition, Input.mousePosition);
    }

    void Shoot(Vector2 center, Vector2 direction)
    {
        float dx = direction.x - center.x,
            dy = direction.y - center.y,
            d = dx * dx + dy * dy;

        if (!(MinMouseRadius * MinMouseRadius <= d) || !(d <= MaxMouseRadius * MaxMouseRadius)) return;

        var shootDirection = (center - direction) / Screen.width * defaultScreen;

        //inAir = true;
        rigidbody.AddForce(shootDirection * Force);
    }

    void OnCollisionEnter2D(Collision2D collision)
    {
        if (collision.gameObject.CompareTag("Platform"))
            inAir = false;
    }

    void OnCollisionExit2D(Collision2D other)
    {
        inAir = true;
    }

    public void OnMovedToPlatform(PlatformController platform)
    {
        inAir = false;
        game.AddScore("Dishes");
        cameraController.StabilizeCamera();
    }

    public void OnKilled()
    {
        // Play some animations
    }
    
    public void StopMoving()
    {
        inAir = false;
        rigidbody.velocity = Vector2.zero;
        rigidbody.angularVelocity = 0f;
    }

    public void OnCollectedFood(FoodControl.Foods food) => game.AddScore(food);
}