using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace DefaultNamespace
{
    public class GameManager : MonoBehaviour
    {
        [Header("Core")] public PlayerController player;
        public Camera camera;
        public Canvas ui;
        [SerializeField] GameObject DeathScreenLayer;
        public GameObject Foods;

        [Header("Game Stats")] public int DishesPassed;
        public GameObject Cheese, Cabbage, Tomato, Dishes;
        public int[] Scores = new int[4], Targets = new int[4];

        FoodScore[] scores;

        void Awake()
        {
            var names = new[] {"Cheese", "Cabbage", "Tomato", "Dishes"};
            scores = new[] {Cheese, Cabbage, Tomato, Dishes}
                .Select((t, i) => new FoodScore {name = names[i], sprite = t, score = Scores[i], target = Targets[i]})
                .ToArray();
        }

        public void KillPlayer()
        {
            DeathScreenLayer.SetActive(true);
            player.OnKilled();
        }

        public void AddScore(string name)
        {
            scores.First(it => it.name == name).score++;
            UpdateScore();
        }

        public void AddScore(FoodControl.Foods food) => AddScore(food.ToString());

        void UpdateScore() =>
            scores
                .ToList()
                .ForEach(it =>
                    it.sprite.transform.GetChild(0).GetComponent<Text>().text =
                        it.target > 0 ? $"{it.score}/{it.target}" : it.score.ToString());

        void OnSandwitchReady()
        {
        }

        public void TryAgain() => SceneManager.LoadScene("SampleScene");
    }

    public class FoodScore
    {
        public string name;
        public GameObject sprite;
        public int score, target;
    }
}