﻿using System.Collections.Generic;
using DefaultNamespace;
using UnityEngine;

public class PlayerController : MonoBehaviour
{
    [SerializeField] GameManager game;
    [SerializeField] float MinMouseRadius = 5f;
    [SerializeField] float MaxMouseRadius = 100f;

    [SerializeField] float Force = 10f;

    Rigidbody2D rigidbody;
    Vector2 startMousePosition;
    CameraControl cameraController;
    bool isAiming, inAir, needToGoThrough;
    
    //TODO опастно для жизни
    readonly Queue<Vector2> lastVelocitys = new Queue<Vector2>();
    public Vector2 lastVelocity;

    void Update()
    {
        lastVelocity = lastVelocitys.Dequeue();
        lastVelocitys.Enqueue(rigidbody.velocity);
    }
    
    void Awake()
    {
        rigidbody = GetComponent<Rigidbody2D>();
        cameraController = game.camera.GetComponent<CameraControl>();
        lastVelocitys.Enqueue(Vector2.zero);
        lastVelocitys.Enqueue(Vector2.zero);
    }

    void OnMouseDown()
    {
        isAiming = true;
        startMousePosition = Input.mousePosition;
    }

    void OnMouseUp()
    {
        isAiming = false;
        
        if (!inAir)
            Shoot(startMousePosition, Input.mousePosition);
    }

    void Shoot(Vector2 center, Vector2 direction)
    {
        float dx = direction.x - center.x,
            dy = direction.y - center.y,
            d = dx * dx + dy * dy;

        Debug.Log(d + " -> " + (MinMouseRadius * MinMouseRadius <= d && d <= MaxMouseRadius * MaxMouseRadius));

        if (!(MinMouseRadius * MinMouseRadius <= d) || !(d <= MaxMouseRadius * MaxMouseRadius)) return;

        var shootDirection = center - direction;

        inAir = true;
        rigidbody.AddForce(shootDirection * Force);
    }

    void OnCollisionEnter2D(Collision2D other)
    {
        inAir = false;
    }

    public void OnMovedToPlatform(PlatformController platform)
    {
        inAir = false;
        cameraController.StabilizeCamera();
    }

    public void OnKilled()
    {
        // Play some animations
    }
    
    public void StopMoving()
    {
        inAir = false;
        rigidbody.velocity = Vector2.zero;
        rigidbody.angularVelocity = 0f;
    }
}